// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Copyright (c) Microsoft Corporation. All rights reserved.

namespace Microsoft.Samples.Commerce.Order.CommerceEntities
{
    using Microsoft.Commerce.Contracts;

    /// <summary>
    /// Represents the cash card payment.
    /// </summary>
    public class CashCard : PaymentAccount
    {
        /// <summary>
        /// defines the model name for this commerce entity
        /// </summary>
        public new const string ModelNameDefinition = "CashCard";

        /// <summary>
        /// Initializes a new instance of the <see cref="CashCard"/> class.
        /// </summary>
        public CashCard() : base(new CommerceEntity(CashCard.ModelNameDefinition))
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CashCard"/> class.
        /// </summary>
        /// <param name="commerceEntity">The commerce entity.</param>
        public CashCard(CommerceEntity commerceEntity)
            : base(commerceEntity)
        {
        }
        
        /// <summary>
        /// Gets or sets the cash card number used for the payment.
        /// </summary>
        /// <value>The cash card number.</value>
        public string CashCardNumber
        {
            get { return this.CommerceEntity.GetPropertyValue(PropertyName.CashCardNumber) as string; }
            set { this.CommerceEntity.SetPropertyValue(PropertyName.CashCardNumber, value); }
        }

        /// <summary>
        /// Gets or sets the users personal information number (PIN).
        /// </summary>
        /// <value>The CashCard pin.</value>
        public string Pin
        {
            get { return this.CommerceEntity.GetPropertyValue(PropertyName.Pin) as string; }
            set { this.CommerceEntity.SetPropertyValue(PropertyName.Pin, value); }
        }

        #region Implicit operators
        /// <summary>
        /// Performs an implicit conversion from <see cref="Microsoft.Commerce.Contracts.CommerceEntity"/> to <see cref="CashCard"/>.
        /// </summary>
        /// <param name="commerceEntity">The commerce entity.</param>
        /// <returns>The result of the conversion.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2225:OperatorOverloadsHaveNamedAlternates", Justification = "Design decision for API")]
        public static implicit operator CashCard(CommerceEntity commerceEntity)
        {
            return new CashCard(commerceEntity);
        }
        #endregion

        /// <summary>
        /// Names of the properties exposed by this commerce entity.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1053:StaticHolderTypesShouldNotHaveConstructors", Justification = "By Design"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1034:NestedTypesShouldNotBeVisible", Justification = "By Design")]
        public new class PropertyName : PaymentAccount.PropertyName
        {
            /// <summary>
            /// The name of the CashCardNumber property for a CashCard. 
            /// </summary>
            public const string CashCardNumber = "CashCardNumber";

            /// <summary>
            /// The name of the Pin property for a CashCard.
            /// </summary>
            public const string Pin = "Pin";
        }      
    }
}
