// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Copyright (c) Microsoft Corporation. All rights reserved.

namespace Microsoft.Samples.Commerce.Order.CommerceEntities
{
    using System;
    using Microsoft.Commerce.Contracts;

    /// <summary>
    /// Represents the gift certificate payment.
    /// </summary>
    public class GiftCertificate : PaymentAccount
    {
        /// <summary>
        /// defines the model name for this commerce entity
        /// </summary>
        public new const string ModelNameDefinition = "GiftCertificate";

        /// <summary>
        /// Initializes a new instance of the <see cref="GiftCertificate"/> class.
        /// </summary>
        public GiftCertificate() : base(new CommerceEntity(GiftCertificate.ModelNameDefinition))
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GiftCertificate"/> class.
        /// </summary>
        /// <param name="commerceEntity">The commerce entity.</param>
        public GiftCertificate(CommerceEntity commerceEntity)
            : base(commerceEntity)
        {
        }

        /// <summary>
        /// Gets or sets the payment authorization code used for the payment.
        /// </summary>
        /// <value>The authorization code.</value>
        public string AuthorizationCode
        {
            get { return this.CommerceEntity.GetPropertyValue(PropertyName.AuthorizationCode) as string; }
            set { this.CommerceEntity.SetPropertyValue(PropertyName.AuthorizationCode, value); }
        }

        /// <summary>
        /// Gets or sets the date on which the gift certificate will expire.
        /// </summary>
        /// <value>The expiration date.</value>
        public DateTime? ExpirationDate
        {
            get { return (DateTime?)this.CommerceEntity.GetPropertyValue(PropertyName.ExpirationDate); }
            set { this.CommerceEntity.SetPropertyValue(PropertyName.ExpirationDate, value ?? null); }
        }

        /// <summary>
        /// Gets or sets the gift certificate number for the current instance.
        /// </summary>
        /// <value>The gift certificate number.</value>
        public string GiftCertificateNumber
        {
            get { return this.CommerceEntity.GetPropertyValue(PropertyName.GiftCertificateNumber) as string; }
            set { this.CommerceEntity.SetPropertyValue(PropertyName.GiftCertificateNumber, value); }
        }

        /// <summary>
        /// Gets or sets the users personal information number (PIN).
        /// </summary>
        /// <value>The gift certificate pin.</value>
        public string Pin
        {
            get { return this.CommerceEntity.GetPropertyValue(PropertyName.Pin) as string; }
            set { this.CommerceEntity.SetPropertyValue(PropertyName.Pin, value); }
        }

        #region Implicit operators
        /// <summary>
        /// Toes the user profile.
        /// </summary>
        /// <param name="commerceEntity">The commerce entity.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2225:OperatorOverloadsHaveNamedAlternates", Justification = "Design decision for API")]
        public static implicit operator GiftCertificate(CommerceEntity commerceEntity)
        {
            return new GiftCertificate(commerceEntity);
        }
        #endregion

        /// <summary>
        /// Names of the properties exposed by this commerce entity.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1053:StaticHolderTypesShouldNotHaveConstructors", Justification = "By Design"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1034:NestedTypesShouldNotBeVisible", Justification = "By Design")]
        public new class PropertyName : PaymentAccount.PropertyName
        {
            /// <summary>
            /// The name of the AuthorizationCode property for a GiftCertificate.
            /// </summary>
            public const string AuthorizationCode = "AuthorizationCode";

            /// <summary>
            /// The name of the ExpirationDate property for a GiftCertificate.
            /// </summary>
            public const string ExpirationDate = "ExpirationDate";

            /// <summary>
            /// The name of the GiftCertificateNumber property for a GiftCertificate.
            /// </summary>
            public const string GiftCertificateNumber = "GiftCertificateNumber";

            /// <summary>
            /// The name of the Pin property for a GiftCertificate.
            /// </summary>
            public const string Pin = "Pin";
        }       
    }
}
