// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Copyright (c) Microsoft Corporation. All rights reserved.

namespace Microsoft.Samples.Commerce.Order.CommerceEntities
{
    using Microsoft.Commerce.Contracts;

    /// <summary>
    /// Payment represents both the amount paid and the payment method used for the order.
    /// </summary>
    public class Payment : ICommerceEntity
    {
        /// <summary>
        /// defines the model name for this commerce entity
        /// </summary>
        public const string ModelNameDefinition = "Payment";

        /// <summary>
        /// The commerce entity.
        /// </summary>
        private CommerceEntity commerceEntity;

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Payment"/> class.
        /// </summary>
        public Payment()
        {
            this.commerceEntity = new CommerceEntity(Payment.ModelNameDefinition);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Payment"/> class.
        /// </summary>
        /// <param name="commerceEntity">The commerce entity.</param>
        public Payment(CommerceEntity commerceEntity)
        {
            this.commerceEntity = commerceEntity;
        }

        #endregion

        #region ICommerceEntity Members

        /// <summary>
        /// Gets or sets the id.
        /// </summary>
        /// <value>The id of the entity.</value>
        public string Id
        {
            get { return this.commerceEntity.Id; }
            set { this.commerceEntity.Id = value; }
        }

        /// <summary>
        /// Gets or sets the type name of this object instance
        /// <remarks>Required so that the Builders can convert to the Multi-channel Commerce Foundation CommerceEntity base type</remarks>
        /// </summary>
        /// <value></value>
        public string ModelName
        {
            get { return this.commerceEntity.ModelName; }
            set { this.commerceEntity.ModelName = value; }
        }

        /// <summary>
        /// Gets or sets this objects collection of Properties and their corresponding values
        /// </summary>
        /// <value></value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly", Justification = "Collection setting is required")]
        public CommercePropertyCollection Properties
        {
            get { return this.commerceEntity.Properties; }
            set { this.commerceEntity.Properties = value; }
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the amount paid.
        /// </summary>
        /// <value>The amount.</value>
        public decimal? Amount
        {
            get { return (decimal?)this.commerceEntity.GetPropertyValue(PropertyName.Amount); }
            set { this.commerceEntity.SetPropertyValue(PropertyName.Amount, value ?? null); }
        }

        /// <summary>
        /// Gets or sets the payment method used.
        /// </summary>
        /// <value>The payment account.</value>
        public CommerceRelationship PaymentAccount
        {
            get { return this.commerceEntity.GetPropertyValue(RelationshipName.PaymentAccount) as CommerceRelationship; }
            set { this.commerceEntity.SetPropertyValue(RelationshipName.PaymentAccount, value); }
        }

        /// <summary>
        /// Gets or sets the status of the payment instance.
        /// </summary>
        /// <value>The status.</value>
        public string Status
        {
            get { return this.commerceEntity.GetPropertyValue(PropertyName.Status) as string; }
            set { this.commerceEntity.SetPropertyValue(PropertyName.Status, value); }
        }

        #endregion

        #region Implicit operators
        /// <summary>
        /// Toes the user profile.
        /// </summary>
        /// <param name="commerceEntity">The commerce entity.</param>
        /// <returns></returns>
        public static implicit operator Payment(CommerceEntity commerceEntity)
        {
            System.Diagnostics.Debug.Assert(Payment.ModelNameDefinition.Equals(commerceEntity.ModelName), "Unexpected model name");
            return new Payment(commerceEntity);
        }
        #endregion

        /// <summary>
        /// Required so that the Builders can convert to the Multi-channel Commerce Foundation CommerceEntity base type using <see cref="ModelName"/>
        /// </summary>
        /// <returns>This instances base type</returns>
        public CommerceEntity ToCommerceEntity()
        {
            return this.commerceEntity;
        }

        #region Property key names

        /// <summary>
        /// Class defining Properties of Items that may be related to an Organization.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1716:IdentifiersShouldNotMatchKeywords", Justification = "Required because of API specification (i.e. DataContract design)")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1034:NestedTypesShouldNotBeVisible", Justification = "Needed for external access"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1053:StaticHolderTypesShouldNotHaveConstructors", Justification = "Cannot make it private as it cannot be derived from")]
        public static class RelationshipName
        {
            /// <summary>
            /// The name of the PaymentAccount property for a Payment.
            /// </summary>
            public const string PaymentAccount = "PaymentAccount";
        }

        /// <summary>
        /// Names of the properties exposed by this commerce entity.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1034:NestedTypesShouldNotBeVisible", Justification = "Needed for external access"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1053:StaticHolderTypesShouldNotHaveConstructors", Justification = "Cannot make it private as it cannot be derived from")]
        public class PropertyName : CommerceEntity.PropertyName
        {
            /// <summary>
            /// The name of the Amount property for a Payment.
            /// </summary>
            public const string Amount = "Amount";

            /// <summary>
            /// The name of the Status property for a Payment.
            /// </summary>
            public const string Status = "Status";
        }

        #endregion
    }
}
